/*
 * Gaim-Rhythmbox plugin.
 *
 * Copyright (C) 2004 Jon Oberheide.
 *
 * $Id: gaim-rhythmbox.c,v 1.29 2005/09/12 18:55:19 binaryjono Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#define GAIM_RHYTHMBOX_PLUGIN_ID	"core-gaim-rhythmbox"
#define RB_IID				"OAFIID:GNOME_Rhythmbox"

#define OPT_GAIMRB 			"/plugins/gaim_rhythmbox"
#define OPT_SHOW_NOT_PLAYING		OPT_GAIMRB "/show_not_playing"
#define OPT_PROCESS_AWAY		OPT_GAIMRB "/process_away"
#define OPT_PROCESS_AVAIL		OPT_GAIMRB "/process_avail"
#define OPT_PROCESS_USERINFO		OPT_GAIMRB "/process_userinfo"

#include "internal.h"
#include "gtkgaim.h"
#include "gtkplugin.h"
#include "aim.h"
#include "aim_internal.h"
#include "rb-proxy.h"
#include "util.h"
#include "away.h"
#include "aim.h"
#include "debug.h"
#include "connection.h"
#include "version.h"

typedef struct {
	GaimPlugin *plugin;
	GList *plugin_deps;
	RbProxy *proxy;
	Bonobo_Listener song_listener;
	Bonobo_Listener playing_listener;
} plugin_t;

plugin_t *data;


static void rb_debug(const gchar *output)
{
	gaim_debug(GAIM_DEBUG_INFO, "Gaim-Rhythmbox", output);
}


static void rb_process_away(GaimConnection *gc, gchar *rb_info)
{
	gchar *away_msg;

	if (gc->away && awaymessage->message) {
		away_msg = gaim_strreplace(awaymessage->message, "%rb", rb_info);

		/* only set the away message if the text has changed */
		if (g_ascii_strcasecmp(awaymessage->message, away_msg) != 0) {
			serv_set_away(gc, GAIM_AWAY_CUSTOM, away_msg);
		}

		g_free(away_msg);
	}
}


static void rb_process_avail(GaimConnection *gc, gchar *rb_info)
{
	gchar *avail_msg;
	OscarData *od = (OscarData *) gc->proto_data;

	/* strip the html since available messages cannot be very long */
	avail_msg = gaim_markup_strip_html(rb_info);

	aim_locate_setprofile(od->sess, NULL, NULL, 0, NULL, "", 0);
	aim_srv_setavailmsg(od->sess, avail_msg);

	g_free(avail_msg);
}


static void rb_process_userinfo(GaimConnection *gc, gchar *rb_info)
{
	gchar *new_userinfo;
	const gchar *userinfo = gaim_account_get_user_info(gaim_connection_get_account(gc));

	if (userinfo) {
		new_userinfo = gaim_strreplace(userinfo, "%rb", rb_info);

		/* only set the user info if the text has changed */
		if (g_ascii_strcasecmp(userinfo, new_userinfo)) {
			serv_set_info(gc, new_userinfo);
		}

		g_free(new_userinfo);
	}
}


static void rb_process(gchar *rb_info)
{
	GList *l;
	GaimConnection *gc;

	if (!gaim_plugin_is_loaded(data->plugin)) {
		return;
	}

	for (l = gaim_connections_get_all(); l != NULL; l = l->next) {
		gc = (GaimConnection *) l->data;

		/* make sure we're connected */
		if (gaim_connection_get_state(gc) != GAIM_CONNECTED) {
			continue;
		}

		/* make sure this connection uses the oscar proto */
		if (g_ascii_strcasecmp(gc->prpl->info->id, "prpl-oscar") != 0) {
			continue;
		}

		if (gaim_prefs_get_bool(OPT_PROCESS_AWAY)) {
			rb_debug("rb_process_away()\n");
			rb_process_away(gc, rb_info);
		}

		if (gaim_prefs_get_bool(OPT_PROCESS_AVAIL)) {
			rb_debug("rb_process_avail()\n");
			rb_process_avail(gc, rb_info);
		}

		if (gaim_prefs_get_bool(OPT_PROCESS_USERINFO)) {
			rb_debug("rb_process_userinfo()\n");
			rb_process_userinfo(gc, rb_info);
		}
	}
}


static void state_changed_cb(RbProxy *proxy, const RbProxySongInfo *song)
{
	gchar *rb_info;
	rb_debug("state_changed_cb()\n");

	if (song) {
		if (g_ascii_strcasecmp(song->artist, "Unknown") == 0) {
			rb_info = g_strdup_printf("%s", song->title);
		} else {
			gchar *url, *query;
			query = g_strdup_printf("lyrics \"%s\" \"%s\"", song->title, song->artist);
			url = g_strdup_printf("http://www.google.com/search?btnI=Im+Feeling+Lucky&q=%s", gaim_url_encode(query));
			rb_info = g_strdup_printf("\"<a href='%s'>%s</a>\" by %s", url, song->title, song->artist);
			g_free(query);
			g_free(url);
		}
	} else {
		if (gaim_prefs_get_bool(OPT_SHOW_NOT_PLAYING)) {
			rb_info = g_strdup("Not Playing");
		} else {
			rb_info = g_strdup("");
		}
	}
	
	rb_process(rb_info);

	g_free(rb_info);
}


static GaimPluginPrefFrame *get_plugin_pref_frame(GaimPlugin *plugin)
{
	GaimPluginPref *pref;
	GaimPluginPrefFrame *frame = gaim_plugin_pref_frame_new();

	pref = gaim_plugin_pref_new_with_label("Gaim-Rhythmbox Configuration");
	gaim_plugin_pref_frame_add(frame, pref);
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_PROCESS_AWAY, "Expand %rb to song info in the away message");
	gaim_plugin_pref_frame_add(frame, pref);
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_PROCESS_USERINFO, "Expand %rb to song info in the user info");
	gaim_plugin_pref_frame_add(frame, pref);
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_PROCESS_AVAIL, "Set available message to song info");
	gaim_plugin_pref_frame_add(frame, pref);
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_SHOW_NOT_PLAYING, "Show \"Not Playing\" message when not playing");
	gaim_plugin_pref_frame_add(frame, pref);

	return frame;
}


static GaimPluginUiInfo pref_info =
{
	get_plugin_pref_frame
};


static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,					/**< type		*/
	GAIM_GTK_PLUGIN_TYPE,					/**< ui_requirement	*/
	0,							/**< flags		*/
	NULL,							/**< dependencies	*/
	GAIM_PRIORITY_DEFAULT,					/**< priority		*/
	GAIM_RHYTHMBOX_PLUGIN_ID,				/**< id			*/
	N_("Gaim-Rhythmbox"),					/**< name		*/
	VERSION,						/**< version		*/
								/**  summary		*/
	N_("Automatically updates your Gaim user info with the currently playing music in Rhythmbox."),
								/**  description	*/
	N_("Automatically updates your Gaim user info with the currently playing music in Rhythmbox."),
	"Jon Oberheide <jon@oberheide.org>",			/**< author		*/
	"http://gaim-rhythmbox.sourceforge.net",		/**< homepage		*/
	NULL,							/**< load		*/
	NULL,							/**< unload		*/
	NULL,							/**< destroy		*/
	NULL,							/**< ui_info		*/
	NULL,							/**< extra_info		*/
	&pref_info,						/**< pref info		*/
	NULL
};


static void init_plugin(GaimPlugin *plugin)
{
	g_type_init();
	data = g_new0(plugin_t, 1);
	data->proxy = rb_proxy_new();
	data->plugin = plugin;

	/* add preferences */
	gaim_prefs_add_none(OPT_GAIMRB);
	gaim_prefs_add_bool(OPT_PROCESS_AWAY, FALSE);
	gaim_prefs_add_bool(OPT_PROCESS_AVAIL, TRUE);
	gaim_prefs_add_bool(OPT_PROCESS_USERINFO, TRUE);
	gaim_prefs_add_bool(OPT_SHOW_NOT_PLAYING, TRUE);

	g_signal_connect(data->proxy, "state_changed", G_CALLBACK(state_changed_cb), data);
}


GAIM_INIT_PLUGIN(gaim_rhythmbox, init_plugin, info)
