/*
 * Implementation of the proxy around Rhythmbox's Bonobo interface.
 *
 * Rhythmbox Applet
 * Copyright (C) 2004 Paul Kuliniewicz <kuliniew@purdue.edu>
 * Modifications by Jon Oberheide <jon@oberheide.org>
 *
 * $Id: rb-proxy.c,v 1.5 2004/09/07 20:21:21 binaryjono Exp $
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <libbonobo.h>
#include "rb-proxy.h"
#include "Rhythmbox.h"

#define POLL_DELAY 10000

struct RbProxyPrivate
{
	GNOME_Rhythmbox rb;
	Bonobo_PropertyBag pb;
	CORBA_Environment ev;
	guint poll;
	Bonobo_Listener song_listener;
	Bonobo_Listener playing_listener;
};

typedef enum
{
	SONG_CHANGED,
	LAST_SIGNAL
} RbProxySignal;

static GObjectClass *parent_class;
static guint rb_proxy_signals[LAST_SIGNAL] = { 0 };

static void rb_proxy_class_init (RbProxyClass *klass);
static void rb_proxy_init (RbProxy *proxy);
static void rb_proxy_finalize (GObject *object);
static void rb_proxy_connect (RbProxy *proxy);
static void rb_proxy_disconnect (RbProxy *proxy);
static gboolean rb_proxy_is_connected (RbProxy *proxy);
static gboolean rb_proxy_poll_cb (RbProxy *proxy);
static void report_error (RbProxy *proxy);
static void state_change_cb (BonoboListener *listener,
			    const char *event_name,
			    const CORBA_any *any,
			    CORBA_Environment *ev,
			    RbProxy *proxy);

GType
rb_proxy_get_type (void)
{
	static GType this_type = 0;

	if (!this_type)
	{
		static const GTypeInfo this_info = {
			sizeof (RbProxyClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_proxy_class_init,
			NULL,
			NULL,
			sizeof (RbProxy),
			0,
			(GInstanceInitFunc) rb_proxy_init,
			NULL
		};
		this_type = g_type_register_static (G_TYPE_OBJECT,
						    "RbProxy",
						    &this_info,
						    0);
	}

	return this_type;
}

static void
rb_proxy_class_init (RbProxyClass *klass)
{
	GObjectClass *object_class = (GObjectClass *) klass;

	parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = rb_proxy_finalize;

	rb_proxy_signals[SONG_CHANGED] =
		g_signal_new ("state_changed",
			      G_OBJECT_CLASS_TYPE (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (RbProxyClass, state_changed),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__POINTER,
			      G_TYPE_NONE,
			      1,
			      G_TYPE_POINTER);
}

static void
rb_proxy_init (RbProxy *proxy)
{
	proxy->priv = g_new0 (RbProxyPrivate, 1);

	if (!bonobo_is_initialized ())
	{
		gint fake_argc = 0;
		bonobo_init (&fake_argc, NULL);
		bonobo_activate ();
	}

	proxy->priv->rb = CORBA_OBJECT_NIL;
	proxy->priv->pb = CORBA_OBJECT_NIL;
	CORBA_exception_init (&proxy->priv->ev);
}

static void
rb_proxy_finalize (GObject *object)
{
	RbProxy *proxy = RB_PROXY (object);

	rb_proxy_disconnect (proxy);
	CORBA_exception_free (&proxy->priv->ev);
	g_free (proxy->priv);

	parent_class->finalize (object);
}

RbProxy *
rb_proxy_new (void)
{
	RbProxy *proxy;

	proxy = g_object_new (RB_TYPE_PROXY, NULL);
	proxy->priv->poll = 0;
	rb_proxy_connect (proxy);

	return proxy;
}

RbProxySongInfo *
rb_proxy_get_song_info (RbProxy *proxy)
{
	GNOME_Rhythmbox_SongInfo *song = NULL;

	g_return_val_if_fail (proxy != NULL, song);

	if (proxy->priv->rb != CORBA_OBJECT_NIL)
	{
		CORBA_any *any = bonobo_pbclient_get_value (
			proxy->priv->pb, "song",
			TC_GNOME_Rhythmbox_SongInfo,
			&proxy->priv->ev);
		if (BONOBO_EX (&proxy->priv->ev))
			rb_proxy_disconnect (proxy);
		if (any != NULL)
		{
			song = (GNOME_Rhythmbox_SongInfo *) any->_value;
			any->_release = FALSE;
			CORBA_free (any);
		}
	}

	return song;
}

static void
rb_proxy_connect (RbProxy *proxy)
{
	if (proxy->priv->poll == 0)
	{
		proxy->priv->poll = g_timeout_add (
			POLL_DELAY, (GSourceFunc) rb_proxy_poll_cb, proxy);
	}

	if (!rb_proxy_is_connected (proxy))
		return;

	proxy->priv->pb = GNOME_Rhythmbox_getPlayerProperties
		(proxy->priv->rb, &proxy->priv->ev);
	if (BONOBO_EX (&proxy->priv->ev))
		report_error (proxy);

	if (proxy->priv->song_listener && proxy->priv->pb != CORBA_OBJECT_NIL)
		bonobo_event_source_client_remove_listener (proxy->priv->pb, proxy->priv->song_listener, &proxy->priv->ev);
	if (BONOBO_EX (&proxy->priv->ev))
		report_error (proxy);
	
	if (proxy->priv->playing_listener && proxy->priv->pb != CORBA_OBJECT_NIL)
		bonobo_event_source_client_remove_listener (proxy->priv->pb, proxy->priv->playing_listener, &proxy->priv->ev);
	if (BONOBO_EX (&proxy->priv->ev))
		report_error (proxy);
	
	proxy->priv->song_listener = bonobo_event_source_client_add_listener_full (
		proxy->priv->pb,
		g_cclosure_new (G_CALLBACK (state_change_cb), proxy, NULL),
		"Bonobo/Property:change:song",
		&proxy->priv->ev);
	if (BONOBO_EX (&proxy->priv->ev))
		report_error (proxy);

	proxy->priv->playing_listener = bonobo_event_source_client_add_listener_full (
		proxy->priv->pb,
		g_cclosure_new (G_CALLBACK (state_change_cb), proxy, NULL),
		"Bonobo/Property:change:playing",
		&proxy->priv->ev);
	if (BONOBO_EX (&proxy->priv->ev))
		report_error (proxy);
}

static void
rb_proxy_disconnect (RbProxy *proxy)
{
	if (proxy->priv->pb != CORBA_OBJECT_NIL)
	{
		bonobo_object_release_unref ((Bonobo_Unknown) proxy->priv->pb,
					     NULL);
		proxy->priv->pb = CORBA_OBJECT_NIL;
	}

	if (proxy->priv->rb != CORBA_OBJECT_NIL)
	{
		proxy->priv->rb = CORBA_OBJECT_NIL;
	}

	g_signal_emit_by_name (proxy, "state_changed", NULL);

	proxy->priv->poll = g_timeout_add (
		POLL_DELAY, (GSourceFunc) rb_proxy_poll_cb, proxy);
}

static gboolean
rb_proxy_is_connected (RbProxy *proxy)
{
	proxy->priv->rb = bonobo_activation_activate (
		 "((repo_ids.has('IDL:GNOME/Rhythmbox:1.0')) AND (_active == TRUE))",
		 NULL, 0, NULL, &proxy->priv->ev);
	if (proxy->priv->rb == CORBA_OBJECT_NIL)
		return (FALSE);
	else
		return (TRUE);
}

static gboolean
rb_proxy_poll_cb (RbProxy *proxy)
{
	rb_proxy_connect (proxy);
	return (TRUE);
}

static void
report_error (RbProxy *proxy)
{
	gchar *what = bonobo_exception_get_text (&proxy->priv->ev);
	g_warning ("Bonobo error: %s", what);
	g_free (what);

	rb_proxy_disconnect (proxy);
}

static void
state_change_cb (BonoboListener *listener,
		const char *event_name,
		const CORBA_any *any,
		CORBA_Environment *ev,
		RbProxy *proxy)
{
	gboolean playing;
	const GNOME_Rhythmbox_SongInfo *rb_song_info;
	
	if (CORBA_TypeCode_equivalent (any->_type, TC_GNOME_Rhythmbox_SongInfo, NULL))
		rb_song_info = (const GNOME_Rhythmbox_SongInfo *) any->_value;
	else if (CORBA_TypeCode_equivalent (any->_type, TC_CORBA_boolean, NULL))
	{
		playing = BONOBO_ARG_GET_BOOLEAN (any);

		if (playing)
			rb_song_info = rb_proxy_get_song_info(proxy);
		else
			rb_song_info = NULL;
	}
	else
	{
		rb_song_info = NULL;
		return;
	}

	g_signal_emit_by_name (proxy, "state_changed", rb_song_info);
}
